"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var guard_1 = require("../shared/guard");
var md5_1 = __importDefault(require("md5"));
var config_1 = __importDefault(require("../config"));
var AzureNamingService = /** @class */ (function () {
    function AzureNamingService() {
    }
    /**
     * Get a resource name for an Azure Service. Naming convention:
     *
     * {prefix}-{shortRegionName}-{shortStageName}(optionally: -{suffix})
     *
     * @param config Serverless Azure Config for service (serverless.service)
     * @param resourceConfig
     * @param suffix
     */
    AzureNamingService.getResourceName = function (config, resourceConfig, suffix) {
        if (resourceConfig && resourceConfig.name) {
            return resourceConfig.name;
        }
        var _a = config.provider, prefix = _a.prefix, region = _a.region, stage = _a.stage;
        var name = [
            prefix,
            this.createShortAzureRegionName(region),
            this.createShortStageName(stage),
        ].join("-");
        if (suffix) {
            name += "-" + suffix;
        }
        return name.toLocaleLowerCase();
    };
    /**
     * Get a name for an Azure resource that is shorter than a max length and has no forbidden characters
     * Naming convention:
     *
     * {safePrefix}{safeRegion}{safeStage}{safeServiceNameHash}
     *
     * @param config Serverless Azure Config for service (serverless.service)
     * @param maxLength Maximum length of name for resource
     * @param resourceConfig Configuration for resource from serverless configuration
     * @param forbidden Regex for characters to remove from name. Defaults to non-alpha-numerics
     * @param replaceWith String to replace forbidden characters. Defaults to empty string
     */
    AzureNamingService.getSafeResourceName = function (config, maxLength, resourceConfig, forbidden, replaceWith) {
        if (forbidden === void 0) { forbidden = /\W+/g; }
        if (replaceWith === void 0) { replaceWith = ""; }
        if (resourceConfig && resourceConfig.name) {
            var name_1 = resourceConfig.name;
            if (name_1.length > maxLength) {
                throw new Error("Name '" + name_1 + "' invalid. Should be shorter than " + maxLength + " characters");
            }
            return name_1.replace(forbidden, replaceWith);
        }
        var _a = config.provider, prefix = _a.prefix, region = _a.region, stage = _a.stage;
        var nameHash = md5_1.default(config.service);
        var safePrefix = prefix.replace(forbidden, replaceWith);
        var safeRegion = this.createShortAzureRegionName(region);
        var safeStage = this.createShortStageName(stage);
        var safeNameHash = nameHash.substr(0, 6);
        var remaining = maxLength - (safePrefix.length + safeRegion.length + safeStage.length + safeNameHash.length);
        // Dynamically adjust the substring based on space needed
        if (remaining < 0) {
            var partLength = Math.floor(Math.abs(remaining) / 3);
            safePrefix = safePrefix.substr(0, partLength);
            safeStage = safeStage.substr(0, partLength);
            safeNameHash = safeNameHash.substr(0, partLength);
        }
        return [safePrefix, safeRegion, safeStage, safeNameHash]
            .join("")
            .toLocaleLowerCase();
    };
    AzureNamingService.getDeploymentName = function (config, timestamp) {
        var maxLength = config_1.default.naming.maxLength.deploymentName;
        var suffix = config_1.default.naming.suffix.deployment;
        var deploymentName = config.provider.deploymentName;
        if (timestamp) {
            maxLength -= timestamp.length + suffix.length;
            ;
            var name_2 = (deploymentName) ? deploymentName.substr(0, maxLength)
                : [AzureNamingService.getSafeResourceName(config, maxLength), suffix].join("-");
            return [name_2, timestamp].join("-");
        }
        return deploymentName.substr(0, maxLength);
    };
    /**
     * Creates a short name to be used for state name abbreviation
     * @param stageName The stage name
     */
    AzureNamingService.createShortStageName = function (stageName) {
        guard_1.Guard.empty(stageName);
        var stageMap = {
            "dogfood": "df",
            "production": "prod",
            "development": "dev",
            "testing": "test"
        };
        return this.createShortName(stageName, stageMap);
    };
    /**
     * Gets the normalized region name from long name (ex. West US 2 -> westus2)
     * @param regionName The region name
     */
    AzureNamingService.getNormalizedRegionName = function (regionName) {
        guard_1.Guard.empty(regionName);
        return regionName.replace(/\W/g, "").toLowerCase();
    };
    /**
     * Creates a short name for an azure region
     * @param regionName The azure region name
     */
    AzureNamingService.createShortAzureRegionName = function (regionName) {
        guard_1.Guard.empty(regionName);
        var locationMap = {
            "north": "n",
            "south": "s",
            "east": "e",
            "west": "w",
            "central": "c",
        };
        return this.createShortName(regionName, locationMap);
    };
    /**
     * Creates a short name from a long name based on a well-known string map
     * @param longName The long name to replace
     * @param wellKnownMap A well known map of long terms to short abbreviations
     */
    AzureNamingService.createShortName = function (longName, wellKnownMap) {
        guard_1.Guard.empty(longName);
        guard_1.Guard.null(wellKnownMap);
        var pattern = "(" + Object.keys(wellKnownMap).join("|") + ")";
        var regex = new RegExp(pattern, "g");
        return longName
            .replace(/\W+/g, "")
            .toLowerCase()
            .split(regex)
            .map(function (part) {
            return wellKnownMap[part] || part.substr(0, 3);
        })
            .join("");
    };
    return AzureNamingService;
}());
exports.AzureNamingService = AzureNamingService;
